/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/

/**
    \file 
        Lighting.h
    \brief
        Shows lighting support in OGRE. Also demonstrates the use of billboards
        and controllers for automatic time-relative behaviour.
*/


#include "ExampleApplication.h"
#include "OgreIlluminationManager.h"
#include "SpriteParticleRenderer.h"


// Listener class for frame updates
class ParticleDemoFrameListener : public ExampleFrameListener
{
protected:
	
	unsigned long framecount;
	SceneNode* lightNode;

public:
    ParticleDemoFrameListener(RenderWindow* window, Camera* maincam, SceneNode* lightNode)
        :ExampleFrameListener(window, maincam) 
				 
    {
		this->lightNode = lightNode;
		framecount = 0;		
    }

    bool frameStarted(const FrameEvent& evt)
    {		
		bool result = ExampleFrameListener::frameStarted(evt);
		
		Timer* timer = Root::getSingleton().getTimer();
		
		float t = (float) timer->getMilliseconds() / 2000.0;
		float r = 150;
		lightNode->setPosition( r * sin(t), r * sin(5 * t) + 200 , r * cos(t));

       framecount++;
        return result;     
    }

	

};

/** Application class */
class ParticleDemoApplication : public ExampleApplication
{

public:

protected:
	SceneNode* kupolaNode;
	SceneNode* mainLightNode;

	void createPostproc()
	{
		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GameTools/Glow");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GameTools/Glow", true);
	}
    
	 virtual void chooseSceneManager(void)
    {
		ExampleApplication::chooseSceneManager();
        // Get the SceneManager, in this case a generic one
       // mSceneMgr = mRoot->createSceneManager("TerrainSceneManager");
    }

	void createScene(void)
    {
		//createPostproc();

		OgreIlluminationManager::getSingleton().setMainCamera(mCamera);
		OgreIlluminationManager::getSingleton().setMainViewport(mWindow->getViewport(0));
		
		//std::string terrain_cfg("terrain.cfg");
		//mSceneMgr -> setWorldGeometry( terrain_cfg );

		Root::getSingleton()._setCurrentSceneManager(mSceneMgr);
		mCamera->setPosition(0,150,100);
		mCamera->setDirection(0,0,-1);
		mCamera->setFOVy(Radian(Degree(80)));
		mCamera->setFarClipDistance(1000);
		// Set ambient light
        mSceneMgr->setAmbientLight(ColourValue(1, 1, 1));

		// Skybox
        mSceneMgr->setSkyBox(true, "Examples/EveningSkyBox", 10);
		SceneNode* rootNode = mSceneMgr->getRootSceneNode();
					
      /*
		//init objects
		Entity* kupola = mSceneMgr->createEntity("kupola", "kupola.mesh");		
				
		//init nodes
		kupolaNode = rootNode->createChildSceneNode( "kupola" );
		kupolaNode->rotate(Vector3(0,1,0),Radian(Degree(90)));
		//kupolaNode->setPosition(0, -150, 0);
		kupolaNode->attachObject( kupola );
		kupolaNode->scale(50,50,50);		
		kupolaNode->_updateBounds();
		MeshPtr kmesh = kupola->getMesh();
		kmesh->buildTangentVectors(0,1);*/

		Entity* head = mSceneMgr->createEntity("head", "athene.mesh");
		SceneNode* headNode = rootNode->createChildSceneNode("head");
		head->setMaterialName("GameTools/Standard");
		headNode->attachObject(head);
		//headNode->setScale(1.5,1.5,1.5);
		AxisAlignedBox bbox = head->getBoundingBox();
		headNode->setPosition(0,80,20);		
		headNode->_updateBounds();

		ParticleSystem* pSys1 = mSceneMgr->createParticleSystem("psys1", "GameTools/Big");
		pSys1->setKeepParticlesInLocalSpace(false);
		SceneNode *pSysNode = rootNode->createChildSceneNode("psys1");
		pSysNode->attachObject(pSys1);
		pSysNode->setPosition(0,0,0);
		
		//Init lights
		Light* mainLight = mSceneMgr->createLight("MainLight");
		mainLight->setType(Light::LT_POINT);
        mainLight->setDiffuseColour(ColourValue::White);
		mainLight->setSpecularColour(ColourValue::White);
		mainLightNode = rootNode->createChildSceneNode();
		mainLightNode->setPosition(0,500,0);
		mainLight->setPosition(mainLightNode->getWorldPosition());
		mainLightNode->attachObject(mainLight); 
		BillboardSet* bbs = mSceneMgr->createBillboardSet("bb", 1);
		bbs->createBillboard(Vector3::ZERO, ColourValue::White);
		bbs->setMaterialName("Flare");
		mainLightNode->attachObject(bbs);

	//	OgreIlluminationManager::getSingleton().createGlobalRun(ILLUMRUN_PHASE_TEXTURE);
	//	OgreIlluminationManager::getSingleton().updateGlobalRun(ILLUMRUN_PHASE_TEXTURE, 1);
	//	OgreIlluminationManager::getSingleton().savePhaseTextureToFile("phase.dds");

		OgreIlluminationManager::getSingleton().initTechniques();
		
   }

	void createFrameListener(void)
    {
		// This is where we instantiate our own frame listener
		mFrameListener= new ParticleDemoFrameListener(mWindow,
											mCamera	, mainLightNode);
       mFrameListener->setPriority(1);
		OgreIlluminationManager::getSingleton().setPriority(2);
		 mRoot->addFrameListener(mFrameListener);
		 mRoot->addFrameListener(&OgreIlluminationManager::getSingleton());
		
    }

};

